%% Copyright (C) 2003, James B. Rawlings and David Q. Mayne
%%
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License as
%% published by the Free Software Foundation; either version 2, or (at
%% your option) any later version.
%%
%% This program is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; see the file COPYING.  If not, write to
%% the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
%% MA 02111-1307, USA.

% Stochastic reaction simulation
% A +B <-> C
% Determine equilibrium via chemical master equation

% Set state vector x: [A;B;C]
Vol = 200;
x = [1;5;0]*Vol;
nmol = x(1);
% Set the reaction rate vector k
k(1) = 1/Vol;
k(2) = 3;
extent = [1:-1/nmol:0]';

% Construct A of the chemical master equation
% dP/dt = A*P
A = zeros(nmol+1,nmol+1);
for i = 1:nmol+1
  n = i-1;
  nu = nmol-n;
  a = n;
  b = x(2)-nu;
  c = nu;
  A(i,i) = -(k(1)*a*b + k(2)*c);
  if (i>1)
    A(i,i-1) = k(2)*(c+1);
  endif
  if (i<nmol+1)
    A(i,i+1) = k(1)*(a+1)*(b+1);
  endif
endfor

v = null(A);
v = v/sum(v);
figure()
plot(v)
##
## set initial condition for p
##
p = zeros(nmol+1,1);
p(nmol+1) = 1*nmol;

deltat = 0.01;
tfinal = 1;
time = [0:deltat:tfinal]';
Ad = expm(A*deltat);
for i = 1:length(time)
  p(:,i+1) = Ad*p(:,i);
  p(:,i+1) = p(:,i+1)/sum(p(:,i+1))*nmol;
endfor

##
## decide on density of extents for plotting
##
elines = 50;
eskip = ceil(nmol/elines);
eindex = [1:eskip:nmol+1];
tlines = 30;
tskip = ceil((length(time)-1)/tlines);
if (nmol < 50)
  tstart = 2;
else
  tstart = 2;
end
tindex = [tstart:tskip:length(time)];
figure()
%clearplot
mesh(time(tindex),extent(eindex),p(eindex,tindex));

%%
%% save output for plotting
%%
lx = length(tindex);
ly = length(eindex);

file = fopen ('prob_dis_large.dat', 'w');
for j = 1:ly
  for i = 1:lx
   fprintf (file, '%f %f %f\n', time(tindex(i)), extent(eindex(j)), p(eindex(j), tindex(i)));
  endfor
  if (j < ly)
    fprintf (file, '\n');
  endif
endfor
fclose (file);